# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2018 Numan Laanait <nlaanait@gmail.com>
# Copyright (C) 2018 Dominik Kriegner <dominik.kriegner@gmail.com>

# Notes:
# 1. Code needs to be tested for multiple layers.
# 2. Code needs to be tested for roughness.

import os

import numpy as np
import xrayutilities as xu
from matplotlib.pylab import *

# Thin-film of (20 nm) CaTiO$_3$/SrTiO$_3$
CTO = xu.materials.CaTiO3
STO = xu.materials.SrTiO3

# Make CTO/STO thin-film.
STO_sub = xu.simpack.Layer(STO, inf)
CTO_layer = xu.simpack.Layer(CTO, 200, roughness=1e-5)
thin_film = xu.simpack.LayerStack('CTO/STO', STO_sub+CTO_layer)

# Initiate dynamical reflectivity model
# Simulating Reflectivity with S,P polarizations @ 1 keV
energy = 1000
model_S = xu.simpack.DynamicalReflectivityModel(thin_film, energy=energy,
                                                polarization='S')
model_P = xu.simpack.DynamicalReflectivityModel(thin_film, energy=energy,
                                                polarization='P')

alpha = np.linspace(0., 90., num=1000)
Reflec_S, Trans_S = model_S.simulate(alpha)
Reflec_P, Trans_P = model_P.simulate(alpha)

# Plots
fig, (ax_P, ax_S) = plt.subplots(1, 2, figsize=(12, 6))

# Reflectance
ax_P.plot(alpha, Reflec_P, label='P-polarization')
ax_P.plot(alpha, Reflec_S, label='S-polarization')
ax_P.legend(loc='upper right')
ax_P.set_xlabel(r'$\alpha$ (deg)')
ax_P.set_ylabel('Reflectivity')
ax_P.set_yscale('log')

# Transmittance
ax_S.plot(alpha, Trans_P, label='P-polarization')
ax_S.plot(alpha, Trans_S, label='S-polarization')
ax_S.legend(loc='upper right')
ax_S.set_xlabel(r'$\alpha$ (deg)')
ax_S.set_ylabel('Transmittance')
# ax_S.set_yscale('log')

suptitle(r'20 nm CaTiO$_3$/SrTiO$_3$, E=%d eV, $\theta_c$=%.2f$^\circ$'
         % (energy, CTO.critical_angle(en=energy)))
tight_layout(rect=(0, 0, 1, 0.94))

# Comparison between Dynamical and Kinematic

# For our simple thin-film, dynamical and kinematic must agree everywhere,
# except for $\alpha \ll \alpha_c$ (strong scattering near angle of total
# external reflection).  There are other conditions where dynamical is more
# accurate (multi-layers, near soft x-ray resonances, etc ...).

xrr_model = xu.simpack.SpecularReflectivityModel(thin_film, energy=energy)
R_kin = xrr_model.simulate(alpha)

# Reflectance
fig, ax = plt.subplots(1, 1, figsize=(12, 6))
ax.plot(alpha, Reflec_S, label='TransferMatrix')
ax.plot(alpha, R_kin, label='SpecularReflectivityModel', linestyle='dashed')
ax.legend(loc='upper right')
ax.set_xlabel(r'$\alpha$ (deg)')
ax.set_ylabel('Reflectance')
ax.set_yscale('log')
ax.set_title(r'20 nm CaTiO$_3$/SrTiO$_3$, E=%d eV, $\theta_c$=%.2f$^\circ$'
             % (energy, STO.critical_angle(en=energy)))
tight_layout()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2015-2020 Dominik Kriegner <dominik.kriegner@gmail.com>

import os.path
import tempfile
import unittest

import h5py
import numpy
import xrayutilities as xu

testfile = 'esrf.edf.gz'
datadir = os.path.join(os.path.dirname(__file__), 'data')
fullfilename = os.path.join(datadir, testfile)


@unittest.skipIf(not os.path.isfile(fullfilename),
                 "additional test data needed (http://xrayutilities.sf.io)")
class TestIO_EDF(unittest.TestCase):
    dshape = (2048, 2048)
    dmax = 60134
    dmin = 72
    tpos = (500, 500)
    dtpos = 2349

    @classmethod
    def setUpClass(cls):
        cls.edffile = xu.io.EDFFile(testfile, path=datadir)
        cls.data = cls.edffile.data

    def test_datashape(self):
        self.assertEqual(self.dshape, self.data.shape)

    def test_datavalues(self):
        self.assertAlmostEqual(self.dmax, self.data.max(), places=10)
        self.assertAlmostEqual(self.dmin, self.data.min(), places=10)
        self.assertAlmostEqual(self.dtpos,
                               self.data[self.tpos[0], self.tpos[1]],
                               places=10)

    def test_savehdf5(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            fname = os.path.join(tmpdir, 'tmp.h5')
            self.edffile.Save2HDF5(fname)
            with h5py.File(fname, 'r') as h5f:
                h5d = h5f[list(h5f.keys())[0]]
                h5d = numpy.asarray(h5d)
                self.assertTrue(numpy.all(h5d == self.data))

    def test_EDFDirectory(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            fname = os.path.join(tmpdir, 'tmp.h5')
            ed = xu.io.EDFDirectory(datadir, 'edf.gz')
            ed.Save2HDF5(fname)
            with h5py.File(fname, 'r') as h5f:
                h5g = h5f[os.path.split(datadir)[-1]]
                h5d = h5g[list(h5g.keys())[0]]
                h5d = numpy.asarray(h5d)
                self.assertTrue(numpy.all(h5d == self.data))


if __name__ == '__main__':
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2016-2020 Dominik Kriegner <dominik.kriegner@gmail.com>

import xrayutilities as xu
from matplotlib.pylab import *
from scipy.special import erf

mpl.rcParams['font.size'] = 16.0
en = 'CuKa1'
qz = linspace(4.3, 4.6, 4000)

GaAs = xu.materials.GaAs
exp = xu.HXRD(GaAs.Q(1, 1, 0), GaAs.Q(0, 0, 1), en=en)
dm = xu.simpack.DarwinModelAlGaAs001(
    qz, experiment=exp, resolution_width=0.0005, I0=2e7, background=1e0)


def period(xavg, xwell, wellratio, thick, sigmaup, sigmadown):
    """
    chemical composition profile in the superlattice
    """
    twell = wellratio*thick
    xb = (xavg*thick - twell*xwell)/(thick - twell)
    tb = (thick-twell)/2
    print('using barrier Al-content, well thickness, barrier thickness/2: '
          '%.3f %.1f %.1f' % (xb, twell, tb))
    return lambda z: xb + (xwell-xb)*(erf((z-tb)/sigmaup) +
                                      erf(-(z-(tb+twell))/sigmadown))/2.


sideal = [{'t': 3500000, 'x': 0, 'r': 1},  # 350um substrate
          {'t': 10000, 'x': 0.5, 'r': 1},  # 1um buffer (relaxed)
          # 5 period superlattice with interdiffusion
          (5, [{'t': 75, 'x': 0.9, 'r': 0},
               {'t': 150, 'x': 0.0, 'r': 0},
               {'t': 75, 'x': 0.9, 'r': 0}]),
          {'t': 100, 'x': 0, 'r': 0}]  # 10nm cap
sample = [{'t': 3500000, 'x': 0, 'r': 1},  # 350um substrate
          {'t': 10000, 'x': 0.5, 'r': 1},  # 1um buffer (relaxed)
          # 5 period superlattice with interdiffusion
          (5, [{'t': 300, 'x': period(0.45, 0, 0.5, 300, 20, 20), 'r': 0.0}]),
          {'t': 100, 'x': 0, 'r': 0}]  # 10nm cap

# perform Darwin-theory based simulation
mlideal = dm.make_monolayers(sideal)
Iideal = dm.simulate(mlideal)
ml = dm.make_monolayers(sample)
Isim = dm.simulate(ml)

figure('XU-simpack AlGaAs (Darwin)', figsize=(10, 5))
clf()

subplot(121)
semilogy(qz, Iideal, '-m', lw=2, label='ideal')
semilogy(qz, Isim, '-r', lw=2, label='SL5')
ylim(0.5*dm.background, dm.I0)
xlim(qz.min(), qz.max())
plt.locator_params(axis='x', nbins=5)
xlabel(r'Qz ($1/\mathrm{\AA}$)')
ylabel('Intensity (arb. u.)')
legend(fontsize='small')

subplot(122)
z, xAl = dm.prop_profile(mlideal, 'x')
plot(z/10, xAl, '-m')
z, xAl = dm.prop_profile(ml, 'x')
plot(z/10, xAl, '-r')
xlabel('depth z (nm)')
ylabel('Al-content', color='m')
ylim(-0.05, 1.05)
twinx()
z, ai = dm.prop_profile(ml, 'ai')
plot(z/10, ai, '-b')
xlim(-1200, 3)
ylim(5.64, 5.68)
plt.locator_params(axis='x', nbins=5)
ylabel(r'a-inplane ($\mathrm{\AA}$)', color='b')
tight_layout()
show()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2012 Dominik Kriegner <dominik.kriegner@gmail.com>

import numpy
# f = f0(|Q|) + f1(en) + j * f2(en)
import xrayutilities as xu

Fe = xu.materials.elements.Fe  # iron atom
Q = numpy.array([0, 0, 1.9], dtype=numpy.double)
en = 10000  # energy in eV

print("Iron (Fe): E: %9.1f eV" % en)
print("f0: %8.4g" % Fe.f0(numpy.linalg.norm(Q)))
print("f1: %8.4g" % Fe.f1(en))
print("f2: %8.4g" % Fe.f2(en))



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2015 Dominik Kriegner <dominik.kriegner@gmail.com>

import os.path
import unittest

import numpy
import xrayutilities as xu

testfile = 'pilatus100K.tif'
datadir = os.path.join(os.path.dirname(__file__), 'data')
fullfilename = os.path.join(datadir, testfile)


@unittest.skipIf(not os.path.isfile(fullfilename),
                 "additional test data needed (http://xrayutilities.sf.io)")
class TestCCD_Normalizer(unittest.TestCase):
    dshape = (2, 195, 487)
    dmax = 0.1
    dmin = -0.2
    nmax = 21

    @classmethod
    def setUpClass(cls):
        imgreader = xu.io.Pilatus100K()
        cls.img = imgreader.readImage(testfile, path=datadir)
        cls.ccddata = numpy.asarray((cls.img, 2*cls.img))
        cls.fakedata = numpy.array([(1.0, 10.0), (2.0, 21.0)],
                                   dtype=[('time', float),
                                          ('moni', float)])
        cls.normalizer = xu.IntensityNormalizer(mon='moni', time='time',
                                                av_mon=1.0)
        cls.normdata = cls.normalizer(cls.fakedata, ccd=cls.ccddata)

    def test_datashape(self):
        self.assertTrue(numpy.all(self.dshape == self.normdata.shape))

    def test_datavalues(self):
        self.assertAlmostEqual(self.dmax, self.normdata.max(), places=10)
        self.assertAlmostEqual(self.dmin, self.normdata.min(), places=10)
        testhist, binedges = numpy.histogram(self.normdata, bins=30)
        self.assertEqual(self.nmax, testhist[-1])


if __name__ == '__main__':
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2009 Eugen Wintersberger <eugen.wintersberger@desy.de>
# Copyright (C) 2009-2019 Dominik Kriegner <dominik.kriegner@gmail.com>

"""
xrayutilities is a Python package for assisting with x-ray diffraction
experiments. Its the python package included in *xrayutilities*.

It helps with planning experiments as well as analyzing the data.

Authors:
 Dominik Kriegner <dominik.kriegner@gmail.com> and
 Eugen Wintersberger <eugen.wintersberger@desy.de>
"""
import os

# load configuration
from . import __path__, analysis, config, io, materials, math, simpack
from .experiment import (GID, GISAXS, HXRD, Experiment, FourC, NonCOP,
                         PowderExperiment, QConversion)
from .gridder import FuzzyGridder1D, Gridder1D, npyGridder1D
from .gridder2d import FuzzyGridder2D, Gridder2D, Gridder2DList
from .gridder3d import FuzzyGridder3D, Gridder3D
from .normalize import (IntensityNormalizer, blockAverage1D, blockAverage2D,
                        blockAverageCCD, blockAveragePSD)
from .q2ang_fit import Q2AngFit
from .utilities import (clear_bit, en2lam, energy, frac2str, lam2en,
                        makeNaturalName, maplog, set_bit, wavelength)

# load package version
with open(os.path.join(__path__[0], 'VERSION')) as version_file:
    __version__ = version_file.read().strip().replace('\n', '.')



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2010-2019 Dominik Kriegner <dominik.kriegner@gmail.com>

"""
xrayutilities utilities contains a conglomeration of useful functions
this part of utilities does not need the config class
"""

import abc
import numbers
import os.path
import re

import numpy
import scipy.constants

from .exception import InputError

try:  # works in Python >3.4
    ABC = abc.ABC
except AttributeError:  # Python 2.7
    ABC = abc.ABCMeta('ABC', (object, ), {'__slots__': ()})


__all__ = ['ABC', 'check_kwargs', 'clear_bit', 'en2lam', 'energies', 'energy',
           'exchange_filepath', 'exchange_path', 'is_valid_variable_name',
           'lam2en', 'makeNaturalName', 'set_bit', 'wavelength']

energies = {
    'CuKa1': 8047.82310,
    'CuKa2': 8027.9117,
    'CuKa12': 8041.18,
    'CuKb': 8905.337,
    'MoKa1': 17479.374,
    'CoKa1': 6930.32,
    'CoKa2': 6915.30}
# wavelength values from International Tables of Crystallography:
# Vol C, 2nd Ed. page 203
# CuKa1: 1.54059292(45) the value in bracket is the uncertainty
# CuKa2: 1.5444140(19)
# CuKa12: mixture 2:1 a1 and a2
# CuKb:  1.392246(14)
# MoKa1: 0.70931713(41)
# Xray data booklet:
# CoKa1
# CoKa2


def set_bit(f, offset):
    """
    sets the bit at an offset
    """
    mask = 1 << offset
    return(f | mask)


def clear_bit(f, offset):
    """
    clears the bet at an offset
    """
    mask = ~(1 << offset)
    return(f & mask)


def lam2en(inp):
    """
    converts the input wavelength in angstrom to an energy in eV

    Parameters
    ----------
    inp :   float or str
        wavelength in angstrom

    Returns
    -------
    float
        energy in eV

    Examples
    --------
    >>> energy = lam2en(1.5406)
    """
    #  E(eV) = h*c/(e * lambda(A)) *1e10
    inp = wavelength(inp)
    c = scipy.constants
    out = c.h * c.speed_of_light / (c.e * inp) * 1e10
    return out


def en2lam(inp):
    """
    converts the input energy in eV to a wavelength in angstrom

    Parameters
    ----------
    inp :   float or str
        energy in eV

    Returns
    -------
    float
        wavlength in angstrom

    Examples
    --------
    >>> wavelength = en2lam(8048)
    """
    #  lambda(A) = h*c/(e * E(eV)) *1e10
    inp = energy(inp)
    c = scipy.constants
    out = c.h * c.speed_of_light / (c.e * inp) * 1e10
    return out


def energy(en):
    """
    convert common energy names to energies in eV

    so far this works with CuKa1, CuKa2, CuKa12, CuKb, MoKa1

    Parameters
    ----------
    en :    float, array-like or str
        energy either as scalar or array with value in eV, which will be
        returned unchanged; or string with name of emission line

    Returns
    -------
    float or array-like
        energy in eV
    """

    if isinstance(en, numbers.Number):
        return numpy.double(en)
    elif isinstance(en, (numpy.ndarray, list, tuple)):
        return numpy.asarray(en)
    elif isinstance(en, str):
        return energies[en]
    else:
        raise InputError("wrong type for argument en")


def wavelength(wl):
    """
    convert common energy names to energies in eV

    so far this works with CuKa1, CuKa2, CuKa12, CuKb, MoKa1

    Parameters
    ----------
    wl :    float, array-like or str
        wavelength; If scalar or array the wavelength in angstrom will be
        returned unchanged, string with emission name is converted to
        wavelength

    Returns
    -------
    float or array-like
        wavelength in angstrom
    """

    if isinstance(wl, numbers.Number):
        return numpy.double(wl)
    elif isinstance(wl, (numpy.ndarray, list, tuple)):
        return numpy.asarray(wl)
    elif isinstance(wl, str):
        return en2lam(energies[wl])
    else:
        raise InputError("wrong type for argument wavelength")


def exchange_path(orig, new, keep=0, replace=None):
    """
    function to exchange the root of a path with the option of keeping the
    inner directory structure. This for example includes such a conversion
    /dir_a/subdir/images/sample -> /home/user/data/images/sample
    where the two innermost directory names are kept (keep=2), or equally
    the three outer most are replaced (replace=3). One can either give keep,
    or replace, with replace taking preference if both are given. Note that
    replace=1 on Linux/Unix replaces only the root for absolute paths.

    Parameters
    ----------
    orig :      str
        original path which should be replaced by the new path
    new :       str
        new path which should be used instead
    keep :      int, optional
        number of inner most directory names which should be kept the same in
        the output (default = 0)
    replace :   int, optional
        number of outer most directory names which should be replaced in the
        output (default = None)

    Returns
    -------
    str
        directory path string

    Examples
    --------
    >>> exchange_path('/dir_a/subdir/img/sam', '/home/user/data', keep=2)
    '/home/user/data/img/sam'
    """
    subdirs = []
    o = orig
    if replace is None:
        for i in range(keep):
            o, s = os.path.split(o)
            subdirs.append(s)
        out = new
        subdirs.reverse()
        for s in subdirs:
            out = os.path.join(out, s)
    else:
        while True:
            o, s = os.path.split(o)
            if not s:
                subdirs.append(o)
                break
            elif not o:
                subdirs.append(s)
                break
            else:
                subdirs.append(s)
        subdirs.reverse()
        out = new
        for s in subdirs[replace:]:
            out = os.path.join(out, s)
    return out


def exchange_filepath(orig, new, keep=0, replace=None):
    """
    function to exchange the root of a filename with the option of keeping the
    inner directory structure. This for example includes such a conversion
    /dir_a/subdir/sample/file.txt -> /home/user/data/sample/file.txt
    where the innermost directory name is kept (keep=1), or equally
    the three outer most are replaced (replace=3). One can either give keep,
    or replace, with replace taking preference if both are given. Note that
    replace=1 on Linux/Unix replaces only the root for absolute paths.

    Parameters
    ----------
    orig :      str
        original filename which should have its data root replaced
    new :       str
        new path which should be used instead
    keep :      int, optional
        number of inner most directory names which should be kept the same in
        the output (default = 0)
    replace :   int, optional
        number of outer most directory names which should be replaced in the
        output (default = None)

    Returns
    -------
    str
        filename string

    Examples
    --------
    >>> exchange_filepath('/dir_a/subdir/sam/file.txt', '/data', 1)
    '/data/sam/file.txt'
    """
    if new:
        if replace is None:
            return exchange_path(orig, new, keep+1)
        else:
            return exchange_path(orig, new, replace=replace)
    else:
        return orig


def makeNaturalName(name, check=False):
    ret = re.sub('[^0-9a-zA-Z]', '_', name.strip())
    isvalid = is_valid_variable_name(ret)
    if not check or isvalid:
        return ret
    elif not isvalid:
        raise ValueError("'{}' is not valid variable name".format(ret))


def is_valid_variable_name(name):
    return name.isidentifier()


def check_kwargs(kwargs, valid_kwargs, identifier):
    """
    Raises an TypeError if kwargs included a key which is not in valid_kwargs.

    Parameters
    ----------
    kwargs :        dict
        keyword arguments dictionary
    valid_kwargs :  dict
        dictionary with valid keyword arguments and their description
    identifier :    str
        string to identifier the caller of this function
    """
    desc = ', '.join(["'%s': %s" % (k, d) for k, d in valid_kwargs.items()])
    for k in kwargs:
        if k not in valid_kwargs:
            raise TypeError("%s: unknown keyword argument ('%s') given; "
                            "allowed are %s" % (identifier, k, desc))



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2013 Raphael Grifone <raphael.grifone@esrf.fr>
# Copyright (C) 2013 Dominik Kriegner <dominik.kriegner@gmail.com>

"""
The function below is a modification of the hotpixelkill function included in
the xrayutilities_id01_functions.py file.

Instead of manually specifying the list of hot pixel they are automatically
determined from a series of dark images. The idea is as follows: If in the
average data of several dark images (no intentional x-ray radiations during
acquisition) a signal above a certain threshold remains it is expected to arise
from a hot pixel and such pixels should be removed to avoid spikes in the data.
"""

import glob

import numpy
import xrayutilities as xu


def hotpixelkill(ccd):
    """
    function to remove hot pixels from CCD frames
    """
    ccd[hotpixelnumbers] = 0
    # one could also use NaN here, since NaN values are
    # ignored by the gridder
    return ccd


# identify hot pixels numbers from a series of dark images
# open a series of 2D detector frames
ccdavg = numpy.empty(0)
n = 0
for f in glob.glob(r"G:\data\dark*.edf"):
    e = xu.io.EDFFile(f)
    ccdraw = e.data
    if len(ccdavg) == 0:
        ccdavg = ccdraw.astype(numpy.float)
    else:
        ccdavg += ccdraw
    n += 1

ccdavg /= float(n)

# adjust treshold value: either as absolute value or in relation to the maximum
threshold = 10  # counts
# or
# threshold = ccdraw.max()*0.1 #take 10% of maximum intensity

# determine hot pixels by comparison with the threshold value
hotpixelnumbers = numpy.where(ccdavg > threshold)



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2015 Dominik Kriegner <dominik.kriegner@gmail.com>

import os.path
import unittest

import xrayutilities as xu

testfile = 'sc.esg'
datadir = os.path.join(os.path.dirname(__file__), 'data')
fullfilename = os.path.join(datadir, testfile)


@unittest.skipIf(not os.path.isfile(fullfilename),
                 "additional test data needed (http://xrayutilities.sf.io)")
class TestIO_ESG(unittest.TestCase):
    dshape = (602, 4096)
    dmax = 69282.0
    dmin = 0.0
    motorname = '_pd_meas_angle_omega'
    motmin = 2.0
    motmax = 20.0
    dtpos = 9.0
    tpos = (200, 600)

    @classmethod
    def setUpClass(cls):
        cls.esgfile = xu.io.pdESG(fullfilename)

    def test_datashape(self):
        self.assertEqual(self.dshape, self.esgfile.data.shape)

    def test_datavalues(self):
        motor = self.esgfile.fileheader[self.motorname]
        counter = self.esgfile.data
        self.assertAlmostEqual(self.motmax, motor.max(), places=6)
        self.assertAlmostEqual(self.motmin, motor.min(), places=6)
        self.assertAlmostEqual(self.dmax, counter.max(), places=6)
        self.assertAlmostEqual(self.dmin, counter.min(), places=6)
        self.assertAlmostEqual(counter[self.tpos[0], self.tpos[1]], self.dtpos)


if __name__ == '__main__':
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2012-2020 Dominik Kriegner <dominik.kriegner@gmail.com>

import matplotlib.pyplot as plt
import numpy
import xrayutilities as xu

matA = xu.materials.InAs
matB = xu.materials.InP
substrate = xu.materials.Si

alloy = xu.materials.CubicAlloy(matA, matB, 0)

hxrd001 = xu.HXRD([1, 1, 0], [0, 0, 1])
qinp, qout = (3.02829203, 4.28265165)

# draw two relaxation triangles for the given Alloy in the substrate
[qxt0, qzt0] = alloy.RelaxationTriangle([2, 2, 4], substrate, hxrd001)
alloy.x = 1.
[qxt1, qzt1] = alloy.RelaxationTriangle([2, 2, 4], substrate, hxrd001)

plt.figure()
plt.plot(qxt0, qzt0, '-r')
plt.plot(qxt1, qzt1, '-b')
plt.plot(qinp, qout, 'ok')

# print concentration of alloy B calculated from a reciprocal space point
print(alloy.ContentBasym(qinp, qout, [2, 2, 4], [0, 0, 1]))



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2014-2018 Dominik Kriegner <dominik.kriegner@gmail.com>

import unittest

import numpy
import xrayutilities as xu


class TestGetAngles(unittest.TestCase):
    chi111 = 70.528779365509308

    @classmethod
    def setUpClass(cls):
        amp = numpy.random.rand()
        fwhm = numpy.random.rand() * 1.5 + 0.1
        cls.x = numpy.arange(-3, 3, 0.0003)
        cls.p = [0., fwhm, amp, 0.]
        cls.p2d = [0., 0., fwhm, fwhm, amp, 0.,
                   2 * numpy.pi * numpy.random.rand()]
        cls.sigma = fwhm / (2 * numpy.sqrt(2 * numpy.log(2)))

    def test_getang111(self):
        chi, phi = xu.analysis.getangles([1, 1, -1], [1, 1, 1], [2, 2, -4])
        self.assertAlmostEqual(chi, self.chi111, places=10)
        self.assertAlmostEqual(phi, 0, places=10)

    def test_getang001(self):
        chi, phi = xu.analysis.getangles([1, 1, 0], [0, 1, 0], [1, 0, 0])
        self.assertAlmostEqual(chi, 45, places=10)
        self.assertAlmostEqual(phi, 0, places=10)
        chi, phi = xu.analysis.getangles([1, 0, 1], [0, 1, 0], [1, 0, 0])
        self.assertAlmostEqual(chi, 90, places=10)
        self.assertAlmostEqual(phi, -45, places=10)

    def test_getunitvector(self):
        hkl = numpy.random.randint(-5, 5, 3)
        while numpy.all(hkl == (0, 0, 0)):
            hkl = numpy.random.randint(-5, 5, 3)
        chi, phi = xu.analysis.getangles(hkl, [1, 1, 1], [1, -1, 0])
        hklvec = xu.analysis.getunitvector(chi, phi, [1, 1, 1], [1, -1, 0])
        for i in range(3):
            self.assertAlmostEqual(hkl[i] / numpy.linalg.norm(hkl),
                                   hklvec[i])


if __name__ == '__main__':
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2012-2019 Dominik Kriegner <dominik.kriegner@gmail.com>

import numpy
import xrayutilities as xu

# definition of materials is done from its space group and its free parameters.
# Atoms in the base are specified by their Wyckoff positions and potential free
# parameters of these positions

# As example we will refine zinc-blende InP which is already predefined as
# xu.materials.InP. Zincblende consists of atoms

# elements (which contain their x-ray optical properties) are loaded from
# xrayutilities.materials.elements
In = xu.materials.elements.In
P = xu.materials.elements.P

# definition of zincblende InP:
InP = xu.materials.Crystal(
    "InP", xu.materials.SGLattice(216, 5.8687, atoms=[In, P],
                                  pos=['4a', '4c']),)

# printing of information about the defined material:
print(InP)

# A very primitive visualization of the unit cell can be performed using
InP.show_unitcell()
# for more sophisticated plotting I suggest you export a CIF file and use the
# software of your choice:
# InP.toCIF('filename.cif')

# for some purposes it might be necessary to convert the SGLattice to space
# group P1
InP_p1 = xu.materials.Crystal("InP (P1)", InP.lattice.convert_to_P1())

print(InP_p1)



"""
example script to show the detector parameter determination for area detectors
from images recorded in the primary beam and at known symmetric coplanar Bragg
reflections of a reference crystal
"""

import os

import numpy
import xrayutilities as xu

Si = xu.materials.Si

datadir = 'data'
specfile = "si_align.spec"

en = 15000  # eV
wl = xu.en2lam(en)
imgdir = os.path.join(datadir, "si_align_")  # data path for CCD files
filetmp = "si_align_12_%04d.edf.gz"

qconv = xu.QConversion(['z+', 'y-'], ['z+', 'y-'], [1, 0, 0])
hxrd = xu.HXRD(Si.Q(1, 1, -2), Si.Q(1, 1, 1), wl=wl, qconv=qconv)

# manually selected images

s = xu.io.SPECFile(specfile, path=datadir)
imagenrs = []
for num in [61, 62, 63, 20, 21, 26, 27, 28]:
    s[num].ReadData()
    imagenrs = numpy.append(imagenrs, s[num].data['ccd_n'])

# avoid images which do not have to full beam on the detector as well as
# other which show signal due to cosmic radiation
avoid_images = [37, 57, 62, 63, 65, 87, 99, 106, 110, 111, 126, 130, 175,
                181, 183, 185, 204, 206, 207, 208, 211, 212, 233, 237, 261,
                275, 290]

images = []
ang1 = []  # outer detector angle
ang2 = []  # inner detector angle
sang = []  # sample rocking angle
hkls = []  # Miller indices of the reference reflections


def hotpixelkill(ccd):
    """
    function to remove hot pixels from CCD frames
    ADD REMOVE VALUES IF NEEDED!
    """
    ccd[304, 97] = 0
    ccd[303, 96] = 0
    return ccd


# read images and angular positions from the data file
# this might differ for data taken at different beamlines since
# they way how motor positions are stored is not always consistent
for imgnr in numpy.sort(list(set(imagenrs) - set(avoid_images))[::4]):
    filename = os.path.join(imgdir, filetmp % imgnr)
    edf = xu.io.EDFFile(filename)
    ccd = hotpixelkill(edf.data)
    images.append(ccd)
    ang1.append(float(edf.header['motor_pos'].split()[4]))
    ang2.append(float(edf.header['motor_pos'].split()[3]))
    sang.append(float(edf.header['motor_pos'].split()[1]))
    if imgnr > 1293.:
        hkls.append((0, 0, 0))
    elif imgnr < 139:
        hkls.append((0, 0, numpy.sqrt(27)))  # (3,3,3))
    else:
        hkls.append((0, 0, numpy.sqrt(75)))  # (5,5,5))

# call the fit for the detector parameters.
# Detector arm rotations and primary beam direction need to be given
# in total 8 detector parameters + 2 additional parameters for the reference
# crystal orientation and the wavelength are fitted, however the 4 misalignment
# parameters of the detector and the 3 other parameters can be fixed.
# The fixable parameters are detector tilt azimuth, the detector tilt angle,
# the detector rotation around the primary beam, the outer angle offset, sample
# tilt, sample tilt azimuth and the x-ray wavelength
# Additionally if accurately known the detector pixel size can be given and
# fixed and instead the detector distance can be fitted.
param, eps = xu.analysis.area_detector_calib_hkl(
    sang, ang1, ang2, images, hkls, hxrd, Si, ['z+', 'y-'], 'x+',
    start=(None, None, 1.0, 45, 1.69, -0.55, -1.0, 1.3, 60., wl),
    fix=(False, False, True, False, False, False, False, False, False, False),
    plot=True)

# Following is an example of the output of the summary of the
# area_detector_calib_hkl function
# total time needed for fit: 624.51sec
# fitted parameters: epsilon: 9.9159e-08 (2,['Parameter convergence'])
# param:
# (cch1,cch2,pwidth1,pwidth2,tiltazimuth,tilt,detrot,outerangle_offset,
# sampletilt,stazimuth,wavelength)
# param: 367.12 349.27 6.8187e-05 6.8405e-05 131.4 2.87 -0.390 -0.061 1.201
# 318.44 0.8254
# please check the resulting data (consider setting plot=True)
# detector rotation axis / primary beam direction (given by user): ['z+', 'y-']
# / x+
# detector pixel directions / distance: z- y+ / 1
#   detector initialization with:
#   init_area('z-', 'y+', cch1=367.12, cch2=349.27, Nch1=516, Nch2=516,
#   pwidth1=6.8187e-05, pwidth2=6.8405e-05, distance=1., detrot=-0.390,
#   tiltazimuth=131.4, tilt=2.867)
# AND ALWAYS USE an (additional) OFFSET of -0.0611deg in the OUTER
# DETECTOR ANGLE!



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2012 Dominik Kriegner <dominik.kriegner@gmail.com>

from math import sqrt

import numpy
import xrayutilities as xu

# define elements
In = xu.materials.elements.In
P = xu.materials.elements.P

# define materials
InP = xu.materials.InP
InPWZ = xu.materials.InPWZ

# approximate version of other hexagonal polytypes
ainp = InP.a
InP4H = xu.materials.Crystal(
    "InP(4H)",
    xu.materials.SGLattice(186, ainp / sqrt(2), 2 * sqrt(4/3.) * ainp,
                           atoms=[In, In, P, P],
                           pos=[('2a', 0), ('2b', 1/4.),
                                ('2a', 3/16.), ('2b', 7/16.)]))

for energy in [8041]:  # eV

    lam = xu.en2lam(energy)  # e in eV -> lam in angstrom
    print('         %d eV = %8.4f A' % (energy, lam))
    print('------------------------------------------------------------------'
          '-----------------')
    print('material |         peak    |   omega  |  2theta  |    phi   |   '
          'tt-om  |     |F|   ')
    print('------------------------------------------------------------------'
          '-----------------')

    exp111 = xu.HXRD(InP.Q(1, 1, -2), InP.Q(1, 1, 1), en=energy)
    exphex = xu.HXRD(InPWZ.Q(1, -1, 0), InPWZ.Q(0, 0, 1), en=energy)

    # InP ZB reflections
    reflections = [[1, 1, 1], [2, 2, 2], [3, 3, 3],
                   [3, 3, 1], [2, 2, 4], [1, 1, 5]]
    mat = InP
    for hkl in reflections:
        qvec = mat.Q(hkl)
        [om, chi, phi, tt] = exp111.Q2Ang(qvec, trans=True)
        F = mat.StructureFactor(qvec, exp111._en)
        F /= mat.lattice.UnitCellVolume()
        print('%8s | %15s | %8.4f | %8.4f | %8.4f | %8.4f | %8.2f '
              % (mat.name, ' '.join(map(str, numpy.round(hkl, 2))), om, tt,
                 phi, tt - om, numpy.abs(F)))

    print('------------------------------------------------------------------'
          '-----------------')
    # InP WZ reflections
    reflections = [[0, 0, 2], [0, 0, 4], [1, -1, 4], [1, -1, 5], [1, -1, 6]]
    mat = InPWZ
    for hkl in reflections:
        qvec = mat.Q(hkl)
        [om, chi, phi, tt] = exphex.Q2Ang(qvec, trans=True)
        F = mat.StructureFactor(qvec, exphex._en)
        F /= mat.lattice.UnitCellVolume()
        print('%8s | %15s | %8.4f | %8.4f | %8.4f | %8.4f | %8.2f '
              % (mat.name, ' '.join(map(str, numpy.round(hkl, 2))), om, tt,
                 phi, tt - om, numpy.abs(F)))

    print('------------------------------------------------------------------'
          '-----------------')
    # InP 4H
    reflections = [[0, 0, 4], [0, 0, 8], [1, -1, 9], [1, -1, 10], [1, -1, 11]]
    mat = InP4H
    for hkl in reflections:
        qvec = mat.Q(hkl)
        [om, chi, phi, tt] = exphex.Q2Ang(qvec, trans=True)
        F = mat.StructureFactor(qvec, exphex._en)
        F /= mat.lattice.UnitCellVolume()
        print('%8s | %15s | %8.4f | %8.4f | %8.4f | %8.4f | %8.2f '
              % (mat.name, ' '.join(map(str, numpy.round(hkl, 2))), om, tt,
                 phi, tt - om, numpy.abs(F)))

    print('------------------------------------------------------------------'
          '-----------------')



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2019 Dominik Kriegner <dominik.kriegner@gmail.com>

import xrayutilities as xu

# very often its useful/needed to get oriented in reciprocal space. The
# following convenience function can visualize the reciprocal space position of
# Bragg peaks and limitations of coplanar diffraction geometry (Laue zoens). If
# you use an interactive matplotlib backend you get the Miller indices of the
# Bragg peak upon mouse movement over the peak and its Bragg angles are printed
# to the console upon clicking. The printed angles correspond to the output of
# Q2Ang of the experimental class. All this is shown below for a substrate/film
# system.

Si = xu.materials.Si
Ge = xu.materials.Ge

geom = xu.HXRD(Si.Q(1, 1, -2), Si.Q(1, 1, 1))

ax, h = xu.materials.show_reciprocal_space_plane(Si, geom, ttmax=160)
ax, h2 = xu.materials.show_reciprocal_space_plane(Ge, geom, ax=ax)

ax.figure.show()

# with default settings only Bragg peaks close to the coplanar plane are shown.
# This can be changed by optional settings 'projection' and 'maxqout'



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2009 Eugen Wintersberger <eugen.wintersberger@desy.de>
# Copyright (C) 2010-2019 Dominik Kriegner <dominik.kriegner@gmail.com>

from . import elements
from .atom import Atom
from .cif import CIFFile, cifexport
from .database import (DataBase, add_f0_from_intertab, add_f0_from_xop,
                       add_f1f2_from_ascii_file, add_f1f2_from_henkedb,
                       add_f1f2_from_kissel, add_mass_from_NIST,
                       init_material_db)
from .material import (Alloy, Amorphous, Crystal, CubicAlloy,
                       CubicElasticTensor, HexagonalElasticTensor, Material,
                       PseudomorphicMaterial, WZTensorFromCub)
from .plot import show_reciprocal_space_plane
from .predefined_materials import *
from .spacegrouplattice import SGLattice, SymOp



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2012 Dominik Kriegner <dominik.kriegner@gmail.com>

"""
Example script to show how to use xrayutilities to read and plot
reciprocal space map scans from a spec file created at the ESRF/ID10B

for details about the measurement see:
    D Kriegner et al. Nanotechnology 22 425704 (2011)
    http://dx.doi.org/10.1088/0957-4484/22/42/425704
"""

import os

import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy
import xrayutilities as xu

# plot settings for matplotlib
mpl.rcParams['font.family'] = 'serif'
mpl.rcParams['font.size'] = 20.0
mpl.rcParams['axes.labelsize'] = 'large'
mpl.rcParams['figure.subplot.bottom'] = 0.16
mpl.rcParams['figure.subplot.left'] = 0.17
mpl.rcParams['savefig.dpi'] = 200
mpl.rcParams['axes.grid'] = False


# global setting for the experiment
sample = "test"  # sample name used also as file name for the data file
en = 8042.5  # x-ray energy in eV
# center channel of the linear detector used in the experiment
center_ch = 715.9
# channels per degree of the linear detector (mounted along z direction,
# which corresponds to twotheta)
chpdeg = 345.28
roi = [100, 1340]  # region of interest of the detector

# intensity normalizer function responsible for count time and absorber
# correction
normalizer_detcorr = xu.IntensityNormalizer(
    "MCA", mon="Monitor", time="Seconds",
    absfun=lambda d: d["detcorr"] / d["psd2"].astype(numpy.float))

# substrate material used for Bragg peak calculation to correct for
# experimental offsets
InP = xu.materials.InP

hxrd = xu.HXRD(InP.Q(1, 1, -2), InP.Q(1, 1, 1), en=en)

# configure linear detector
hxrd.Ang2Q.init_linear('z-', center_ch, 1500., chpdeg=chpdeg, roi=roi)

# read spec file and save to HDF5-file
# since reading is much faster from HDF5 once the data are transformed
h5file = os.path.join("data", sample + ".h5")
try:
    # try if spec file object already exist from a previous run of the script
    # ("run -i" in ipython)
    s
except NameError:
    s = xu.io.SPECFile(sample + ".spec.bz2", path="data")
else:
    s.Update()
s.Save2HDF5(h5file)

#################################
# InP (333) reciprocal space map
omalign = 43.0529  # experimental aligned values
ttalign = 86.0733
# nominal values of the substrate peak
[omnominal, _, _, ttnominal] = hxrd.Q2Ang(InP.Q(3, 3, 3))

# read the data from the HDF5 file (scan number:36, names of motors in
# spec file: omega= sample rocking, gamma = twotheta)
[om, tt], MAP = xu.io.geth5_scan(h5file, 36, 'omega', 'gamma')
# normalize the intensity values (absorber and count time corrections)
psdraw = normalizer_detcorr(MAP)
# remove unusable detector channels/regions (no averaging of detector channels)
psd = xu.blockAveragePSD(psdraw, 1, roi=roi)

# determine offset of substrate peak from experimental values (optional)
omalign, ttalign, p, cov = xu.analysis.fit_bragg_peak(
    om, tt, psd, omalign, ttalign, hxrd, plot=False)

# convert angular coordinates to reciprocal space + correct for offsets
[qx, qy, qz] = hxrd.Ang2Q.linear(om, tt, delta=[omalign - omnominal,
                                                ttalign - ttnominal])

# calculate data on a regular grid of 200x201 points
gridder = xu.Gridder2D(200, 201)
gridder(qy, qz, psd)
INT = xu.maplog(gridder.data.transpose(), 8.5, 0)

# plot the intensity as contour plot
plt.figure()
plt.clf()
cf = plt.contourf(gridder.xaxis, gridder.yaxis, INT, 100, extend='min')
plt.xlabel(r'$Q_{[11\bar2]}$ ($\mathrm{\AA}^{-1}$)')
plt.ylabel(r'$Q_{[\bar1\bar1\bar1]}$ ($\mathrm{\AA}^{-1}$)')
cb = plt.colorbar(cf)
cb.set_label(r"$\log($Int$)$ (cps)")


# The same again for a second (asymmetric peak)

# reciprocal space map around the InP (224)
omalign = 59.550  # experimental aligned values
ttalign = 80.099
# nominal values of the substrate peak
[omnominal, _, _, ttnominal] = hxrd.Q2Ang(InP.Q(2, 2, 4))

# read the data from the HDF5 file (scan number:36, names of motors in
# spec file: omega= sample rocking, gamma = twotheta)
[om, tt], MAP = xu.io.geth5_scan(h5file, (33, 34, 35), 'omega', 'gamma')
# normalize the intensity values (absorber and count time corrections)
psdraw = normalizer_detcorr(MAP)
# remove unusable detector channels/regions (no averaging of detector channels)
psd = xu.blockAveragePSD(psdraw, 1, roi=roi)

# determine offset of substrate peak from experimental values (optional)
omalign, ttalign, p, cov = xu.analysis.fit_bragg_peak(
    om, tt, psd, omalign, ttalign, hxrd, plot=False)

# convert angular coordinates to reciprocal space + correct for offsets
[qx, qy, qz] = hxrd.Ang2Q.linear(om, tt, delta=[omalign - omnominal,
                                                ttalign - ttnominal])

# calculate data on a regular grid of 400x600 points
gridder = xu.Gridder2D(400, 600)
gridder(qy, qz, psd)
# calculate intensity which should be plotted
INT = xu.maplog(gridder.data.transpose(), 8.5, 0)

# plot the intensity as contour plot
plt.figure()
plt.clf()
cf = plt.contourf(gridder.xaxis, gridder.yaxis, INT, 100, extend='min')
# levels = numpy.logspace(numpy.log10(INT[INT>0].min()/2.),
#                         numpy.log10(INT.max()),num=100)
# norm = mpl.colors.LogNorm() should be used in future (not yet working in
# matplotlib with extend='min')
plt.xlabel(r'$Q_{[11\bar2]}$ ($\mathrm{\AA}^{-1}$)')
plt.ylabel(r'$Q_{[\bar1\bar1\bar1]}$ ($\mathrm{\AA}^{-1}$)')
cb = plt.colorbar(cf)
cb.set_label(r"$\log($Int$)$ (cps)")

# clean up HDF5 file (not needed in real life!)
os.remove(h5file)



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2018 Dominik Kriegner <dominik.kriegner@gmail.com>

import math
import unittest

import numpy
import xrayutilities as xu


class Test_analysis_linecuts(unittest.TestCase):
    exp = xu.HXRD([1, 0, 0], [0, 0, 1])
    qmax = (2 * exp.k0) / math.sqrt(2) - 0.1
    qyp, qzp = numpy.sort(numpy.random.rand(2) * (qmax-2) + 2)
    width1 = 0.015
    width2 = 0.002
    qy = numpy.linspace(qyp-0.1, qyp+0.1, 601)
    qz = numpy.linspace(qzp-0.1, qzp+0.1, 617)
    Ncut = 450
    QY, QZ = numpy.meshgrid(qy, qz)
    omp, _, _, ttp = exp.Q2Ang(0, qyp, qzp, trans=False)

    def test_radial_cut(self):
        omegaang = math.radians(self.omp-self.ttp/2.)
        data = xu.math.Gauss2d(self.QY, self.QZ, self.qyp, self.qzp,
                               self.width1, self.width2, 1, 0, omegaang)
        x, d, m = xu.analysis.get_radial_scan(
            [self.QY, self.QZ], data, (self.qyp, self.qzp), self.Ncut, 1,
            intdir='2theta')
        x2, d2, m2 = xu.analysis.get_radial_scan(
            [self.QY, self.QZ], data, (self.qyp, self.qzp), self.Ncut, 1,
            intdir='omega')
        self.assertEqual(x.size, self.Ncut)
        self.assertTrue(xu.math.fwhm_exp(x, d) > xu.math.fwhm_exp(x2, d2))

    def test_omega_cut(self):
        radialang = math.radians((self.omp-self.ttp/2.) - 90)
        data = xu.math.Gauss2d(self.QY, self.QZ, self.qyp, self.qzp,
                               self.width1, self.width2, 1, 0, radialang)
        x, d, m = xu.analysis.get_omega_scan(
            [self.QY, self.QZ], data, (self.qyp, self.qzp), self.Ncut, 1,
            intdir='2theta')
        x2, d2, m2 = xu.analysis.get_omega_scan(
            [self.QY, self.QZ], data, (self.qyp, self.qzp), self.Ncut, 1,
            intdir='radial')
        self.assertEqual(x.size, self.Ncut)
        self.assertTrue(xu.math.fwhm_exp(x, d) > xu.math.fwhm_exp(x2, d2))

    def test_ttheta_cut(self):
        omegaang = math.radians(self.omp-self.ttp/2.)
        data = xu.math.Gauss2d(self.QY, self.QZ, self.qyp, self.qzp,
                               self.width1, self.width2, 1, 0, omegaang)
        x, d, m = xu.analysis.get_ttheta_scan(
            [self.QY, self.QZ], data, (self.qyp, self.qzp), self.Ncut, 1,
            intdir='radial')
        x2, d2, m2 = xu.analysis.get_ttheta_scan(
            [self.QY, self.QZ], data, (self.qyp, self.qzp), self.Ncut, 1,
            intdir='omega')
        self.assertEqual(x.size, self.Ncut)
        self.assertTrue(xu.math.fwhm_exp(x, d) > xu.math.fwhm_exp(x2, d2))

    def test_qz_cut(self):
        omegaang = math.radians(self.omp-self.ttp/2.)
        data = xu.math.Gauss2d(self.QY, self.QZ, self.qyp, self.qzp,
                               self.width1, self.width2, 1, 0, omegaang)
        x, d, m = xu.analysis.get_qz_scan(
            [self.QY, self.QZ], data, self.qyp, self.Ncut, 1, intdir='2theta')
        x2, d2, m2 = xu.analysis.get_qz_scan(
            [self.QY, self.QZ], data, self.qyp, self.Ncut, 1, intdir='omega')
        self.assertEqual(x.size, self.Ncut)
        self.assertTrue(xu.math.fwhm_exp(x, d) > xu.math.fwhm_exp(x2, d2))

    def test_qy_cut(self):
        omegaang = math.radians(self.omp-self.ttp/2.)
        data = xu.math.Gauss2d(self.QY, self.QZ, self.qyp, self.qzp,
                               self.width1, self.width2, 1, 0, omegaang)
        x, d, m = xu.analysis.get_qy_scan(
            [self.QY, self.QZ], data, self.qzp, self.Ncut, 1, intdir='2theta')
        x2, d2, m2 = xu.analysis.get_qy_scan(
            [self.QY, self.QZ], data, self.qzp, self.Ncut, 1, intdir='omega')
        self.assertTrue(xu.math.fwhm_exp(x, d) > xu.math.fwhm_exp(x2, d2))

    def test_qcut_width(self):
        data = xu.math.Gauss2d(self.QY, self.QZ, self.qyp, self.qzp,
                               self.width1, self.width2, 1, 0, 0)
        x, d, m = xu.analysis.get_qz_scan(
            [self.QY, self.QZ], data, self.qyp, self.Ncut, 0.1,
            intdir='q')
        self.assertEqual(x.size, self.Ncut)
        self.assertAlmostEqual(
            xu.math.fwhm_exp(x, d) / (2*math.sqrt(2*math.log(2))),
            self.width2, places=4)
        x, d, m = xu.analysis.get_qy_scan(
            [self.QY, self.QZ], data, self.qzp, self.Ncut, 0.1,
            intdir='q')
        self.assertAlmostEqual(
            xu.math.fwhm_exp(x, d) / (2*math.sqrt(2*math.log(2))),
            self.width1, places=4)

    def test_arbitrary_cut(self):
        data = xu.math.Gauss2d(self.QY, self.QZ, self.qyp, self.qzp,
                               self.width1, self.width2, 1, 0, 0)
        x, d, m = xu.analysis.get_arbitrary_line(
            [self.QY, self.QZ], data, (self.qyp, self.qzp), (1, 0), self.Ncut,
            0.01)
        x2, d2, m2 = xu.analysis.get_arbitrary_line(
            [self.QY, self.QZ], data, (self.qyp, self.qzp), (0, 1), self.Ncut,
            0.01)

        self.assertEqual(x.size, self.Ncut)
        self.assertAlmostEqual(x[numpy.argmax(d)], self.qyp, places=2)
        self.assertAlmostEqual(x2[numpy.argmax(d2)], self.qzp, places=2)
        self.assertAlmostEqual(
            xu.math.fwhm_exp(x, d) / (2*math.sqrt(2*math.log(2))),
            self.width1, places=4)
        self.assertAlmostEqual(
            xu.math.fwhm_exp(x2, d2) / (2*math.sqrt(2*math.log(2))),
            self.width2, places=4)


if __name__ == '__main__':
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2016 Dominik Kriegner <dominik.kriegner@gmail.com>

import glob
import os.path

from .. import config
from .helper import xu_h5open


class FileDirectory(object):

    """
    Parses a directory for files, which can be stored to a HDF5 file for
    further usage. The file parser is given to the constructor and must provide
    a Save2HDF5 method.
    """

    def __init__(self, datapath, ext, parser, **keyargs):
        """

        Parameters
        ----------
        datapath :  str
            directory of the files
        ext :       str
            extension of the files in the datapath
        parser :    class
            Parser class for the data files.
        keyargs :   dict
            further keyword arguments are passed to the constructor of the
            parser
        """

        self.datapath = os.path.normpath(datapath)
        self.extension = ext
        self.parser = parser

        # create list of files to read
        self.files = glob.glob(os.path.join(
            self.datapath, '*.%s' % (self.extension)))

        if not self.files:
            print("XU.io.FileDirectory: no file found in %s" % (self.datapath))
            return

        if config.VERBOSITY >= config.INFO_ALL:
            print("XU.io.FileDirectory: %d files found in %s"
                  % (len(self.files), self.datapath))

        self.init_keyargs = keyargs

    def Save2HDF5(self, h5f, group="", comp=True):
        """
        Saves the data stored in the found files in the specified directory in
        a HDF5 file as a HDF5 arrays in a subgroup.  By default the data is
        stored in a group given by the foldername - this can be changed by
        passing the name of a target group or a path to the target group via
        the "group" keyword argument.

        Parameters
        ----------
        h5f :   file-handle or str
            a HDF5 file object or name
        group : str, optional
            group where to store the data (defaults to pathname if group is
            empty string)
        comp :  bool, optional
            activate compression - true by default
        """
        with xu_h5open(h5f, 'a') as h5:
            if isinstance(group, str):
                if group == "":
                    group = os.path.split(self.datapath)[1]
                g = h5.get(group)
                if not g:
                    g = h5.create_group(group)
            else:
                g = group

            for infile in self.files:
                # read EDFFile and save to hdf5
                filename = os.path.split(infile)[1]
                e = self.parser(filename, path=self.datapath,
                                **self.init_keyargs)
                e.Save2HDF5(h5, group=g, comp=comp)



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2015 Dominik Kriegner <dominik.kriegner@gmail.com>

import unittest

import xrayutilities as xu


class TestStructureFactor(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.at = xu.materials.elements.Dummy
        cls.mat = xu.materials.Crystal(
            'test', xu.materials.SGLattice('227:1', 4, atoms=[cls.at, ],
                                           pos=['8a', ]))

    def test_StructureFactor(self):
        f = self.mat.StructureFactor(self.mat.Q(1, 3, 1))
        self.assertAlmostEqual(f, 4 - 4j, places=10)
        f = self.mat.StructureFactor(self.mat.Q(0, 4, 0))
        self.assertAlmostEqual(f, 8, places=10)
        f = self.mat.StructureFactor(self.mat.Q(1, 2, 1))
        self.assertAlmostEqual(f, 0, places=10)

    def test_StructureFactorQ(self):
        q = (self.mat.Q(1, 1, 1), self.mat.Q(0, 4, 0), self.mat.Q(1, 2, 1))
        f = self.mat.StructureFactorForQ(q)
        for i in range(3):
            self.assertAlmostEqual(f[i], (4 + 4j, 8, 0)[i], places=10)

    def test_StructureFactorE(self):
        q = self.mat.Q(1, 1, 1)
        f = self.mat.StructureFactorForEnergy(q, (1000, 2000, 3000))
        for i in range(3):
            self.assertAlmostEqual(f[i], 4 + 4j, places=10)


if __name__ == '__main__':
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2013 Dominik Kriegner <dominik.kriegner@gmail.com>

"""
script to show how the detector parameters (like pixel width, center channel
and detector tilt can be determined for a linear detector.
"""

import os

import matplotlib.pyplot as plt
import xrayutilities as xu

en = xu.utilities.energies["CuKa1"]  # eV
dfile = os.path.join("data", "primarybeam_alignment20130403_2_dis350.nja")

# read seifert file
s = xu.io.SeifertScan(dfile)
ang = s.axispos["T"]
spectra = s.data[:, :, 1]

pwidth, cch, tilt = xu.analysis.linear_detector_calib(ang, spectra)



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2019 Dominik Kriegner <dominik.kriegner@gmail.com>

import xrayutilities as xu
from matplotlib.pylab import *

# global parameters
wavelength = xu.wavelength('CuKa1')
offset = -0.035  # angular offset of the zero position of the data

# set up LayerStack for simulation: InAs(001)/(In,Mn)As(~25 nm)
InAs = xu.materials.InAs
InAs.lattice.a = 6.057
lInAs = xu.simpack.Layer(InAs, inf)
InMnAs = xu.materials.Crystal('InMnAs', xu.materials.SGLattice(
    216, 6.050, atoms=('In', 'Mn', 'As'), pos=('4a', '4a', '4c'),
    occ=(0.88, 0.12, 1)), cij=InAs.cij)
lInMnAs = xu.simpack.Layer(InMnAs, 254, relaxation=0)
pstack = xu.simpack.PseudomorphicStack001('list', lInAs, lInMnAs)

# set up simulation object
thetaMono = arcsin(wavelength/(2 * xu.materials.Ge.planeDistance(2, 2, 0)))
Cmono = cos(2 * thetaMono)
dyn = xu.simpack.DynamicalModel(pstack, I0=1.5e9, background=0,
                                resolution_width=2e-3, polarization='both',
                                Cmono=Cmono)
fitmdyn = xu.simpack.FitModel(dyn)
fitmdyn.set_param_hint('InMnAs_c', vary=True, min=6.02, max=6.06)
fitmdyn.set_param_hint('InAs_a', vary=True)
fitmdyn.set_param_hint('InMnAs_a', expr='InAs_a')
fitmdyn.set_param_hint('resolution_width', vary=True)
params = fitmdyn.make_params()

# plot experimental data
f = figure(figsize=(7, 5))
d = xu.io.RASFile('inas_layer_radial_002_004.ras.bz2', path='data')
scan = d.scans[-1]
tt = scan.data[scan.scan_axis] - offset
semilogy(tt, scan.data['int'], 'o-', ms=3, label='data')

# perform fit and plot the result
fitmdyn.lmodel.set_hkl((0, 0, 4))
ai = (d.scans[-1].data[d.scan.scan_axis] - offset)/2
fitr = fitmdyn.fit(d.scans[-1].data['int'], params, ai)
print(fitr.fit_report())

simdyn = fitmdyn.eval(fitr.params, x=tt/2)
plot(tt, simdyn, label='sim')

# label the plot
ylabel('Intensity (counts)')
xlabel('2Theta (deg)')
legend()
ylim(2e1, 1e9)
xlim(57, 65)
tight_layout()
show()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2018 Dominik Kriegner <dominik.kriegner@gmail.com>

import unittest

import numpy
import xrayutilities as xu


class Test_DiffuseReflectivityModel(unittest.TestCase):
    dmax = 2e-6
    # define used layer stack
    sub = xu.simpack.Layer(xu.materials.Si, numpy.inf, roughness=1,
                           lat_correl=100)
    lay1 = xu.simpack.Layer(xu.materials.Si, 200, roughness=1, lat_correl=200)
    lay2 = xu.simpack.Layer(xu.materials.Ge, 70, roughness=3, lat_correl=50)

    ls = xu.simpack.LayerStack('SL 5', sub+5*(lay2+lay1))

    # simulation parameters
    kwargs = dict(sample_width=10, beam_width=1, energy='CuKa1',
                  vert_correl=1000, vert_nu=0, H=1, method=1, vert_int=0)

    @classmethod
    def setUpClass(cls):
        cls.m1 = xu.simpack.DiffuseReflectivityModel(cls.ls, **cls.kwargs)
        cls.kwargs['H'] = cls.kwargs['H'] - xu.config.EPSILON
        cls.m2 = xu.simpack.DiffuseReflectivityModel(cls.ls, **cls.kwargs)
        cls.kwargs['H'] = 1
        cls.kwargs['method'] = 2
        cls.m3 = xu.simpack.DiffuseReflectivityModel(cls.ls, **cls.kwargs)
        cls.ai = numpy.arange(0.3, 2, 0.005)

    def test_Calculation(self):
        sim = self.m1.simulate(self.ai)
        self.assertEqual(len(sim), len(self.ai))
        self.assertTrue(numpy.all(sim >= 0))

    def test_Consistency(self):
        # calc models
        sim1 = self.m1.simulate(self.ai)
        sim2 = self.m2.simulate(self.ai)
        sim3 = self.m3.simulate(self.ai)

        self.assertTrue(abs(numpy.mean(sim1) - numpy.mean(sim2)) < self.dmax)
        self.assertTrue(abs(numpy.mean(sim1) - numpy.mean(sim3)) < self.dmax)


if __name__ == '__main__':
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2014 Dominik Kriegner <dominik.kriegner@gmail.com>

import numpy
import xrayutilities as xu

xu.config.VERBOSITY = 3

x = numpy.linspace(10, 20, 250)
y = (xu.math.PseudoVoigt1d(x, 12, 0.2, 0.1, 10, numpy.random.rand()) +
     0.01 * (numpy.random.rand(len(x))-0.5))

# peaktype can be 'PseudoVoigt', 'Lorentz' or 'Gauss'
# furthermore a constant background (optional also linear with the option
# background='linear') can be added
xu.math.peak_fit(x, y, peaktype='PseudoVoigt', plot=True)



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2015 Dominik Kriegner <dominik.kriegner@gmail.com>

import unittest

import numpy
import xrayutilities as xu


class Test_maplog(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.dmin = xu.config.DYNLOW
        cls.dmax = xu.config.DYNHIGH

    def test_maplog(self):
        d = numpy.logspace(1, 6, 100)
        # make noop
        dm = xu.maplog(d, numpy.inf, -numpy.inf)
        self.assertTrue(numpy.all(numpy.log10(d) == dm))
        # cut bottom
        dl = 3
        dm = xu.maplog(d, dl, -numpy.inf)
        dl = min(dl, self.dmin)
        self.assertAlmostEqual(d.max(), 10.0**dm.max(), places=10)
        self.assertAlmostEqual(d.max()/10.0**dl, 10**dm.min(), places=10)
        # cut top
        dt = 2
        dm = xu.maplog(d, numpy.inf, dt)
        dt = max(dt, self.dmax)
        self.assertAlmostEqual(d.min(), 10.0**dm.min(), places=10)
        self.assertAlmostEqual(d.max()/10.0**dt, 10.0**dm.max(), places=10)

    def test_maplogzero(self):
        d = numpy.array((0, 1))
        # make function call with a zero and negative number
        dm = xu.maplog(d)
        self.assertAlmostEqual(d.max()/10.0**self.dmax,
                               10.0**dm.max(), places=10)
        self.assertAlmostEqual(d.max()/10.0**self.dmin,
                               10**dm.min(), places=10)
        # call with negative number
        d[0] = -1
        dm = xu.maplog(d)
        self.assertAlmostEqual(d.max()/10.0**self.dmax,
                               10.0**dm.max(), places=10)
        self.assertAlmostEqual(d.max()/10.0**self.dmin,
                               10**dm.min(), places=10)


if __name__ == '__main__':
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2017 Dominik Kriegner <dominik.kriegner@gmail.com>

import multiprocessing
import os

import lmfit
import numpy
import xrayutilities as xu
from matplotlib.pylab import *


def main():
    """dummy main function to enable multiprocessing on windows"""
    mpl.rcParams['font.size'] = 16.0
    mpl.rcParams['lines.linewidth'] = 2.0

    ###############################
    # load example data
    tt, det, sig = loadtxt(os.path.join('data', 'LaB6_d500_si_psd.xye.bz2'),
                           unpack=True)

    ##############################
    La = xu.materials.elements.La
    B = xu.materials.elements.B
    LaB6 = xu.materials.Crystal(
        "LaB6", xu.materials.SGLattice(221, 4.15692, atoms=[La, B],
                                       pos=['1a', ('6f', 0.19750)],
                                       b=[0.05, 0.15]))

    LaB6_powder = xu.simpack.Powder(LaB6, 1,
                                    crystallite_size_gauss=1e6,
                                    crystallite_size_lor=0.5e-6,
                                    strain_gauss=0,
                                    strain_lor=0)

    settings = {'classoptions': {'oversampling': 10},
                'global': {'diffractometer_radius': 0.337,
                           'equatorial_divergence_deg': 0.40},
                'tube_tails': {'tail_left': -0.001,
                               'main_width': 0.00015,
                               'tail_right': 0.001,
                               'tail_intens': 0.0015},
                'axial': {'angI_deg': 2.0, 'angD_deg': 2.0,
                          'slit_length_target': 0.008,
                          'n_integral_points': 21,
                          'length_sample': 0.015,
                          'slit_length_source': 0.008001},
                'si_psd': {'si_psd_window_bounds': (0, 32e-3)},
                'absorption': {'sample_thickness': 500e-6,
                               'absorption_coefficient': 3e4},
                'displacement': {'specimen_displacement': -3.8e-5,
                                 'zero_error_deg': 0.0},
                'emission': {'emiss_intensities': (1.0, 0.45)}}

    pm = xu.simpack.PowderModel(LaB6_powder, I0=1.10e6, fpsettings=settings)

    ##############################
    # define background
    btt, bint = asarray([(15.158, 1136.452),
                         (17.886, 841.925),
                         (22.906, 645.784),
                         (26.556, 551.663),
                         (34.554, 401.219),
                         (45.764, 260.595),
                         (58.365, 171.993),
                         (81.950, 112.838),
                         (92.370, 101.276),
                         (106.441, 102.486),
                         (126.624, 112.838),
                         (139.096, 132.063),
                         (146.240, 136.500),
                         (152.022, 157.204)]).T

    pm.set_background('spline', x=btt, y=bint)
    mask = numpy.logical_and(tt > 18, tt < 148)

    ##############################
    # first fit run to optimize sample displacement and zero offset
    p = pm.create_fitparameters()
    for pn, limit in (('primary_beam_intensity', (None, None)),
                      ('displacement_specimen_displacement', (-1e-4, 1e-4)),
                      ('displacement_zero_error_deg', (-0.01, 0.01))):
        p[pn].set(vary=True, min=limit[0], max=limit[1])

    fitres1 = pm.fit(p, tt[mask], det[mask], std=sig[mask], maxfev=50)

    ##############################
    # second fit run to optimize absorption
    p = pm.create_fitparameters()
    for pn, limit in (('primary_beam_intensity', (None, None)),
                      ('displacement_specimen_displacement', (-1e-4, 1e-4)),
                      ('absorption_absorption_coefficient', (1e4, 10e4)),
                      ('emission_emiss_intensities_item_1', (0.4, 0.52))):
        p[pn].set(vary=True, min=limit[0], max=limit[1])

    # if needed one can also set relations between parameters:
    # p['axial_angD_deg'].set(expr='axial_angI_deg')

    fitres2 = pm.fit(p, tt[mask], det[mask], std=sig[mask])

    ##############################
    # final calculation and plotting/printing of the results
    sim = pm.simulate(tt[mask])

    xu.simpack.plot_powder(tt, det, pm, scale='sqrt', mask=mask)
    xu.simpack.Rietveld_error_metrics(det[mask], sim, std=sig[mask],
                                      Nvar=fitres2.nvarys, disp=True)

    lmfit.report_fit(fitres2)


if __name__ == '__main__':
    multiprocessing.freeze_support()
    main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2013 Eugen Wintersberger <eugen.wintersberger@desy.de>
# Copyright (C) 2013 Dominik Kriegner <dominik.kriegner@gmail.com>


import os

import numpy
import vtk
import xrayutilities as xu
from vtk.util import numpy_support

import xrayutilities_id01_functions as id01

home = "DATADIR"  # data path (root)
datadir = os.path.join(home, "FOLDERNAME")  # data path for CCD/Maxipix files
specdir = home  # location of spec file

sample = "SAMPLENAME"  # sample name -> used as spec file name
# template for the CCD file names
ccdfiletmp = os.path.join(datadir, "CCDFILENAME_%04d.edf.gz")


h5file = os.path.join(specdir, sample + ".h5")
# read spec file and save to HDF5 (needs to be done only once)
# try: s
# except NameError: s = xu.io.SPECFile(sample+".spec",path=specdir)
# else:
#    s.Update()
# s.Save2HDF5(h5file)
# in ipython run with: "run -i script" to just update the spec file and parse
# for new scans only

# number of points to be used during the gridding
nx, ny, nz = 100, 101, 102

# read and grid data with helper function
qx, qy, qz, gint, gridder = id01.gridmap(h5file, SCANNR, ccdfiletmp,
                                         nx, ny, nz)

# prepare data for export to VTK image file
INT = xu.maplog(gint, 3.0, 0)

# export variables qx, qy, qz, INT
qx0 = qx.min()
dqx = (qx.max() - qx.min()) / nx

qy0 = qy.min()
dqy = (qy.max() - qy.min()) / ny

qz0 = qz.min()
dqz = (qz.max() - qz.min()) / nz

INT = numpy.transpose(INT).reshape((INT.size))
data_array = numpy_support.numpy_to_vtk(INT)

image_data = vtk.vtkImageData()

if vtk.vtkVersion.GetVTKMajorVersion() < 6:
    image_data.SetNumberOfScalarComponents(1)
    image_data.SetScalarTypeToDouble()
else:
    vtkinfo = image_data.GetInformation()
    image_data.SetPointDataActiveScalarInfo(vtkinfo, vtk.VTK_DOUBLE, 1)

image_data.SetOrigin(qx0, qy0, qz0)
image_data.SetSpacing(dqx, dqy, dqz)
image_data.SetExtent(0, nx - 1, 0, ny - 1, 0, nz - 1)


pd = image_data.GetPointData()
pd.SetScalars(data_array)

# export data to file
writer = vtk.vtkXMLImageDataWriter()
writer.SetFileName("output.vti")
if vtk.vtkVersion.GetVTKMajorVersion() < 6:
    writer.SetInput(image_data)
else:
    writer.SetInputData(image_data)

writer.Write()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2017 Dominik Kriegner <dominik.kriegner@gmail.com>

import os.path
import unittest

import numpy
import xrayutilities as xu

xu.config.VERBOSITY = 0
testfile = 'rg5041sup1.cif'
datadir = os.path.join(os.path.dirname(__file__), 'data')
fullfilename = os.path.join(datadir, testfile)


@unittest.skipIf(not os.path.isfile(fullfilename),
                 "additional test data needed (http://xrayutilities.sf.io)")
class TestMAT_CIF(unittest.TestCase):
    sg = '129:2'
    mata = 3.820
    matc = 6.318

    @classmethod
    def setUpClass(cls):
        cls.mat = xu.materials.Crystal.fromCIF(fullfilename)

    def test_spacegroup(self):
        self.assertEqual(self.mat.lattice.space_group, self.sg)

    def test_latticeparameters(self):
        self.assertAlmostEqual(self.mat.lattice.a, self.mata, places=6)
        self.assertAlmostEqual(self.mat.lattice.c, self.matc, places=6)

    def test_structurefactor(self):
        Q = self.mat.Q(numpy.random.randint(-3, 4, size=3))
        self.assertAlmostEqual(self.mat.StructureFactor(Q),
                               xu.materials.CuMnAs.StructureFactor(Q),
                               places=3)


if __name__ == '__main__':
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2015 Dominik Kriegner <dominik.kriegner@gmail.com>

import os.path
import unittest

import numpy
import xrayutilities as xu

xu.config.VERBOSITY = 0  # make no output during test
testfile = 'seifert.nja.bz2'
datadir = os.path.join(os.path.dirname(__file__), 'data')
fullfilename = os.path.join(datadir, testfile)


@unittest.skipIf(not os.path.isfile(fullfilename),
                 "additional test data needed (http://xrayutilities.sf.io)")
class TestIO_NJA(unittest.TestCase):
    dshape = (501, 2)
    dmax = 1344706.25
    dmin = 90.0
    motmin = -0.05
    motmax = 0.15
    axisT = 1.5
    dtpos = [5.00000000e-02, 5.64250000e+03]
    tpos = 250

    @classmethod
    def setUpClass(cls):
        cls.njafile = xu.io.SeifertScan(testfile, path=datadir)

    def test_datashape(self):
        self.assertEqual(self.dshape, self.njafile.data.shape)

    def test_headervalue(self):
        self.assertAlmostEqual(self.axisT,
                               self.njafile.axispos['T'][0], places=6)

    def test_datavalues(self):
        motor = self.njafile.data[:, 0]
        counter = self.njafile.data[:, 1]
        self.assertAlmostEqual(self.motmax, motor.max(), places=6)
        self.assertAlmostEqual(self.motmin, motor.min(), places=6)
        self.assertAlmostEqual(self.dmax, counter.max(), places=6)
        self.assertAlmostEqual(self.dmin, counter.min(), places=6)
        self.assertTrue(numpy.all(self.dtpos == self.njafile.data[self.tpos]))


if __name__ == '__main__':
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2017 Dominik Kriegner <dominik.kriegner@gmail.com>

import os
import unittest
from multiprocessing import freeze_support

import numpy
import xrayutilities as xu

try:
    import lmfit
except ImportError:
    lmfit = None

testfile = 'LaB6_d500_si_psd.xye.bz2'
datadir = os.path.join(os.path.dirname(__file__), 'data')
fullfilename = os.path.join(datadir, testfile)


@unittest.skipIf(not os.path.isfile(fullfilename) or lmfit is None,
                 "additional test data (see http://xrayutilities.sf.io) and "
                 "the lmfit Python package are needed")
class Test_PowderModel(unittest.TestCase):
    chi2max = 1.5
    # define powder material
    La = xu.materials.elements.La
    B = xu.materials.elements.B
    LaB6 = xu.materials.Crystal(
        "LaB6", xu.materials.SGLattice(221, 4.15692, atoms=[La, B],
                                       pos=['1a', ('6f', 0.19750)],
                                       b=[0.05, 0.15]))
    LaB6_powder = xu.simpack.Powder(LaB6, 1,
                                    crystallite_size_gauss=1e6,
                                    crystallite_size_lor=0.5e-6,
                                    strain_gauss=0,
                                    strain_lor=0)
    # machine settings
    settings = {'classoptions': {'oversampling': 10},
                'global': {'diffractometer_radius': 0.337,
                           'equatorial_divergence_deg': 0.40},
                'tube_tails': {'tail_left': -0.001,
                               'main_width': 0.00015,
                               'tail_right': 0.001,
                               'tail_intens': 0.0015},
                'axial': {'angI_deg': 2.0, 'angD_deg': 2.0,
                          'slit_length_target': 0.008,
                          'n_integral_points': 21,
                          'length_sample': 0.015,
                          'slit_length_source': 0.008001},
                'si_psd': {'si_psd_window_bounds': (0, 32e-3)},
                'absorption': {'sample_thickness': 500e-6,
                               'absorption_coefficient': 3e4},
                'displacement': {'specimen_displacement': -3.8e-5,
                                 'zero_error_deg': 0.0},
                'emission': {'emiss_intensities': (1.0, 0.45)}}
    # define background
    btt, bint = numpy.asarray([(15.158, 1136.452),
                               (17.886, 841.925),
                               (22.906, 645.784),
                               (26.556, 551.663),
                               (34.554, 401.219),
                               (45.764, 260.595),
                               (58.365, 171.993),
                               (81.950, 112.838),
                               (92.370, 101.276),
                               (106.441, 102.486),
                               (126.624, 112.838),
                               (139.096, 132.063),
                               (146.240, 136.500),
                               (152.022, 157.204)]).T

    def setUp(self):
        with xu.io.xu_open(fullfilename) as fid:
            self.tt, self.det, self.sig = numpy.loadtxt(fid, unpack=True)
        self.mask = numpy.logical_and(self.tt > 18, self.tt < 148)
        self.pm = xu.simpack.PowderModel(self.LaB6_powder, I0=1.10e6,
                                         fpsettings=self.settings)
        self.pm.set_background('spline', x=self.btt, y=self.bint)

    def test_Calculation(self):
        x = numpy.arange(10, 140+numpy.random.rand()*10,
                         0.0075+numpy.random.rand()*0.005)
        sim = self.pm.simulate(x)
        self.pm.close()
        self.assertEqual(len(sim), len(x))
        self.assertTrue(numpy.all(sim >= 0))

    def test_multiprocessing(self):
        x = numpy.arange(10, 140+numpy.random.rand()*10,
                         0.0075+numpy.random.rand()*0.005)
        st_sim = self.pm.simulate(x, mode='local')
        mt_sim = self.pm.simulate(x, mode='multi')
        self.pm.close()
        self.assertAlmostEqual(numpy.sum(numpy.abs(st_sim - mt_sim)), 0.0)

    def test_fitting(self):
        # first fit run
        p = self.pm.create_fitparameters()
        for pn, limit in (
                ('primary_beam_intensity', (None, None)),
                ('displacement_specimen_displacement', (-1e-4, 1e-4)),
                ('displacement_zero_error_deg', (-0.01, 0.01))):
            p[pn].set(vary=True, min=limit[0], max=limit[1])

        fitres1 = self.pm.fit(p, self.tt[self.mask], self.det[self.mask],
                              std=self.sig[self.mask], maxfev=50)

        # second fit run to optimize absorption
        p = fitres1.params
        for pn, limit in (
                ('primary_beam_intensity', (None, None)),
                ('displacement_specimen_displacement', (-1e-4, 1e-4)),
                ('absorption_absorption_coefficient', (1e4, 10e4))):
            p[pn].set(vary=True, min=limit[0], max=limit[1])

        fitres2 = self.pm.fit(p, self.tt[self.mask], self.det[self.mask],
                              std=self.sig[self.mask])
        fitsim = self.pm.simulate(self.tt[self.mask])
        M, Rp, Rwp, Rwpexp, chi2 = xu.simpack.Rietveld_error_metrics(
            self.det[self.mask], fitsim, std=self.sig[self.mask],
            Nvar=fitres2.nvarys, disp=False)
        self.pm.close()

        self.assertTrue(chi2 < self.chi2max)


if __name__ == '__main__':
    freeze_support()
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2018 Dominik Kriegner <dominik.kriegner@gmail.com>

import time

import xrayutilities as xu
from matplotlib.pylab import *

sub = xu.simpack.Layer(xu.materials.Si, inf, roughness=1, lat_correl=100)
lay1 = xu.simpack.Layer(xu.materials.Si, 200, roughness=1, lat_correl=200)
lay2 = xu.simpack.Layer(xu.materials.Ge, 70, roughness=3, lat_correl=50)

ls = xu.simpack.LayerStack('SL 5', sub+5*(lay2+lay1))

alphai = arange(0.17, 2, 0.001)

print("calculate method=1, H=1, vert=0")
start = time.time()
m = xu.simpack.DiffuseReflectivityModel(ls, sample_width=10, beam_width=1,
                                        energy='CuKa1', vert_correl=1000,
                                        vert_nu=0, H=1, method=1, vert_int=0)
d1 = m.simulate(alphai)
print("elapsed time: %.4f" % (time.time() - start))


print("calculate method=2, H=1, vert=0")
start = time.time()
m = xu.simpack.DiffuseReflectivityModel(ls, sample_width=10, beam_width=1,
                                        energy='CuKa1', vert_correl=1000,
                                        vert_nu=0, H=1, method=2, vert_int=0)
d2 = m.simulate(alphai)
print("elapsed time: %.4f" % (time.time() - start))

figure()
semilogy(alphai, d1, label='method=1')
semilogy(alphai, d2, label='method=2')

legend()
xlabel('incidence angle (deg)')
ylabel('intensity (arb. u.)')
tight_layout()



"""
example script to show how the detector parameters
such as pixel width, center channel and detector tilt
can be determined for a linear detector.
"""

import os

import xrayutilities as xu

# load any data file with with the detector spectra of a reference scan
# in the primary beam, here I use spectra measured with a Seifert XRD
# diffractometer
dfile = os.path.join("data", "primarybeam_alignment20130403_2_dis350.nja")
s = xu.io.SeifertScan(dfile)

ang = s.axispos["T"]  # detector angles during the scan
spectra = s.data[:, :, 1]  # detector spectra aquired

# determine detector parameters
# this function accepts some optional arguments to describe the goniometer
# see the API documentation
pwidth, cch, tilt = xu.analysis.linear_detector_calib(ang, spectra,
                                                      usetilt=True)



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2019 Dominik Kriegner <dominik.kriegner@gmail.com>

import os
import subprocess
import sys
import tempfile
import unittest

scriptdir = os.path.join(os.path.dirname(os.path.realpath(__file__)), '..',
                         'examples')
scriptfiles = [
    'simpack_powdermodel.py',
    'simpack_xrd_AlGaAs.py',
    'simpack_xrd_Darwin_AlGaAs.py',
    'simpack_xrd_InAs_fitting.py',
    'simpack_xrd_SiGe111.py',
    'simpack_xrd_SiGe_asymmmetric.py',
    'simpack_xrd_SiGe.py',
    'simpack_xrd_SiGe_superlattice.py',
    'simpack_xrr_diffuse.py',
    'simpack_xrr_matrixmethod.py',
    'simpack_xrr_SiO2_Ru_CoFe_IrMn_Al2O3.py',
    'xrayutilities_angular2hkl_conversion.py',
    # 'xrayutilities_ccd_parameter.py',  # data file not included
    'xrayutilities_components_of_the_structure_factor.py',
    'xrayutilities_define_material.py',
    'xrayutilities_energy_dependent_structure_factor.py',
    # 'xrayutilities_example_plot_3D_ESRF_ID01.py',  # data file not included
    'xrayutilities_experiment_angle_calculation.py',
    'xrayutilities_experiment_kappa.py',
    'xrayutilities_experiment_Powder_example_Iron.py',
    # 'xrayutilities_export_data2vtk.py',  # needs vtk + data
    'xrayutilities_fuzzygridding.py',
    'xrayutilities_hotpixelkill_variant.py',
    'xrayutilities_id01_functions.py',
    'xrayutilities_io_cif_parser_bi2te3.py',
    'xrayutilities_io_cif_parser.py',
    # 'xrayutilities_io_pdcif_plot.py',  # data file not included
    # 'xrayutilities_kmap_example_ESRF.py',  # data file not included
    # 'xrayutilities_linear_detector_parameters.py',  # data file not included
    'xrayutilities_materials_Alloy_contentcalc.py',
    'xrayutilities_math_fitting.py',
    'xrayutilities_orientation_matrix.py',
    'xrayutilities_peak_angles_beamtime.py',
    # 'xrayutilities_polefigure.py',  # basemap needed
    'xrayutilities_q2ang_general.py',
    'xrayutilities_read_panalytical.py',
    # 'xrayutilities_read_seifert.py',  # data file not included
    'xrayutilities_read_spec.py',
    'xrayutilities_reflection_strength.py',
    'xrayutilities_show_reciprocal_space_plane.py',
]


class TestExampleScriptsMeta(type):
    def __new__(mcs, name, bases, dict):
        def test_generator(scriptname):
            def test(self):
                with tempfile.TemporaryFile(mode='w') as fid:
                    env = os.environ.copy()
                    env['MPLBACKEND'] = 'agg'
                    cmd = [sys.executable, scriptname]
                    subprocess.run(cmd, env=env, cwd=scriptdir, stdout=fid,
                                   check=True)
            return test

        for sf in scriptfiles:
            test_name = 'test_%s' % os.path.splitext(sf)[0]
            test = test_generator(sf)
            dict[test_name] = test
        return type.__new__(mcs, name, bases, dict)


@unittest.skipIf(sys.version_info < (3, 5),
                 "needs subprocess.run -> python 3.5 or newer")
class TestExampleScripts(unittest.TestCase, metaclass=TestExampleScriptsMeta):
    pass


if __name__ == '__main__':
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2015-2020 Dominik Kriegner <dominik.kriegner@gmail.com>

import os
import unittest

import numpy
import xrayutilities as xu


def getimage(fname, hotpixelmap, roi=None):
    cbf = xu.io.CBFFile(fname)
    ccdr = removehotpixel(cbf.data, hotpixelmap)
    ccd = xu.blockAverage2D(ccdr, 1, 1, roi=roi)
    if ccd.max() / ccd.mean() < 1e2:
        print('no clear maximum in %s?' % fname)
    ccd[ccd < 2 * ccd.mean()] = 0  # make center of mass position easier
    return ccd


def removehotpixel(ccd, hotpixelmap):
    """
    Removes hotpixel of a ccd image.  Replaces the hotpixel by the mean value
    of the sourrounding non-hotpixel-pixels.

    Parameters
    ----------
     ccd: ccd image stored in 2D numpy array
     hotpixelmap: a tupel with two numpy arrays with the respective x and y
                  coordinates of the hotpixels.

    Returns
    -------
        2D numpy array with the removed hotpixel image data
    """
    shape = ccd.shape
    ccdb = numpy.ones((shape[0] + 2, shape[1] + 2))
    ccdb[1:-1, 1:-1] = ccd

    hotpixelimage = numpy.zeros(shape)
    hotpixelimage[hotpixelmap] = 1

    hotpixelimageb = numpy.zeros((shape[0] + 2, shape[1] + 2))
    hotpixelimageb[1:-1, 1:-1] = hotpixelimage

    y = hotpixelmap[0]
    x = hotpixelmap[1]

    directions = [(0, 1), (0, -1), (1, 1), (1, -1)]
    neighbors = numpy.zeros(len(directions))
    for xc, yc in zip(x, y):
        c = [yc + 1, xc + 1]
        neighbors[...] = 0

        for j, item in enumerate(directions):
            while hotpixelimageb[c[0], c[1]] != 0:
                c[item[0]] += item[1]
            neighbors[j] = ccdb[c[0], c[1]]
            c = [yc + 1, xc + 1]
        ccd[yc, xc] = neighbors.mean()
    return ccd


datadir = os.path.join(os.path.dirname(__file__), 'data')
name = "align"
datfile = os.path.join(datadir, name, name + "_%05d.dat")
ccdfile = os.path.join(datadir, name, name + "_%05d_eiger",
                       name + "_%05d_eiger_%05d.cbf")
fullfilename = ccdfile % (2, 2, 104)


@unittest.skipIf('CI' in os.environ or not os.path.isfile(fullfilename),
                 "this test is not running on CI or additional test "
                 "data are needed (http://xrayutilities.sf.io)")
class TestArea_calib(unittest.TestCase):
    en = 10000.0  # x-ray energy in eV
    roi = (551, 1065, 0, 1030)  # get data of the good part of the detector
    hkls = ((0, 0, 0), (0, 0, 0), (0, 0, 4), (0, 0, 4), (0, 0, 2), (0, 0, 2))
    step = 8  # take only every 8'th point
    slices = (numpy.s_[108::step], numpy.s_[::step],
              numpy.s_[57::step], numpy.s_[::step],
              numpy.s_[56:95:step], numpy.s_[::step])

    @classmethod
    def setUpClass(cls):
        hotpix = xu.io.CBFFile(fullfilename)
        cls.hotpixelmap = numpy.where(hotpix.data > 1)

    def test_area_calib(self):
        scannrs = (2, 4)
        ang1, ang2 = [numpy.empty(0), ] * 2
        r = self.roi
        detdata = numpy.empty((0, r[1] - r[0], r[3] - r[2]))

        for scannr, scanhkl, sl in zip(scannrs, self.hkls, self.slices):
            (tth, tt), angles = xu.io.gettty08_scan(datfile, scannr,
                                                    'tth', 'tt')
            intensity = numpy.zeros((len(angles[sl]),
                                     r[1] - r[0], r[3] - r[2]))
            # read images
            for i, nr in enumerate(angles[sl]['Number']):
                fname = ccdfile % (scannr, scannr, nr)
                intensity[i] = getimage(fname, self.hotpixelmap, roi=self.roi)

            ang1 = numpy.concatenate((ang1, tth[sl]))
            ang2 = numpy.concatenate((ang2, tt[sl]))
            detdata = numpy.concatenate((detdata, intensity))

        # start calibration
        param, eps = xu.analysis.sample_align.area_detector_calib(
            ang1, ang2, detdata, ['z+', 'y-'], 'x+',
            debug=False, plot=False)

        self.assertTrue(True)

    def test_area_calib_hkl(self):
        scannrs = (2, 4, 43, 44, 46, 47)
        sang, ang1, ang2 = [numpy.empty(0), ]*3
        imghkl = numpy.empty((0, 3))
        r = self.roi
        detdata = numpy.empty((0, r[1] - r[0], r[3] - r[2]))

        for scannr, scanhkl, sl in zip(scannrs, self.hkls, self.slices):
            (om, tth, tt), angles = xu.io.gettty08_scan(datfile, scannr,
                                                        'om', 'tth', 'tt')
            intensity = numpy.zeros((len(angles[sl]),
                                     r[1] - r[0], r[3] - r[2]))
            # read images
            for i, nr in enumerate(angles[sl]['Number']):
                fname = ccdfile % (scannr, scannr, nr)
                intensity[i] = getimage(fname, self.hotpixelmap, roi=self.roi)

            if scanhkl == (0, 0, 0):
                sang = numpy.concatenate((sang, (0,)*len(angles[sl])))
            else:
                sang = numpy.concatenate((sang, om[sl]))
            ang1 = numpy.concatenate((ang1, tth[sl]))
            ang2 = numpy.concatenate((ang2, tt[sl]))
            detdata = numpy.concatenate((detdata, intensity))
            imghkl = numpy.concatenate((imghkl, (scanhkl,)*len(angles[sl])))

        # start calibration
        GaAs = xu.materials.GaAs
        qconv = xu.experiment.QConversion(['z+', 'y-', 'x+', 'z-'],
                                          ['z+', 'y-'], [1, 0, 0])
        hxrd = xu.HXRD(GaAs.Q(1, 1, 0), GaAs.Q(0, 0, 1),
                       en=self.en, qconv=qconv)

        param, eps = xu.analysis.sample_align.area_detector_calib_hkl(
            sang, ang1, ang2, detdata, imghkl, hxrd, GaAs,
            ['z+', 'y-'], 'x+',
            start=(None, None, 1., 0, 0, 0, 0, 0, 0, xu.en2lam(self.en)),
            debug=False, plot=False)

        self.assertTrue(True)


if __name__ == '__main__':
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2013 Dominik Kriegner <dominik.kriegner@gmail.com>

import os

import matplotlib.pyplot as plt
import xrayutilities as xu

# global setting for the experiment
sample = "testnja"  # sample name used also as file name for the data file

hxrd = xu.HXRD((1, 1, 0), (0, 0, 1))

#################################
# read the data from the Seifert NJA files
om, tt, psd = xu.io.getSeifert_map(sample + '_%02d.nja', [3, 4], path="data")

# convert angular coordinates to reciprocal space + correct for offsets
[qx, qy, qz] = hxrd.Ang2Q(om, tt)

# calculate data on a regular grid of 200x201 points
gridder = xu.Gridder2D(200, 600)
gridder(qy, qz, psd)
INT = xu.maplog(gridder.data.transpose(), 6, 0)

# plot the intensity as contour plot
plt.figure()
plt.clf()
cf = plt.contourf(gridder.xaxis, gridder.yaxis, INT, 100, extend='min')
plt.xlabel(r'$Q_{[110]}$ ($\mathrm{\AA}^{-1}$)')
plt.ylabel(r'$Q_{[001]}$ ($\mathrm{\AA}^{-1}$)')
cb = plt.colorbar(cf)
cb.set_label(r"$\log($Int$)$ (cps)")



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2012-2018 Dominik Kriegner <dominik.kriegner@gmail.com>

# ALSO LOOK AT THE FILE xrayutilities_id01_functions.py

import os

import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy
import xrayutilities as xu

import xrayutilities_id01_functions as id01

sample = 'SAMPLENAME'  # here used for the specfilename
try:
    s
except NameError:
    s = xu.io.SPECFile(sample + ".spec", path=id01.datadir)
else:
    # in ipython run with: "run -i script" to just update the spec file and
    # parse for new scans only
    s.Update()

# number of points to be used during the gridding
nx, ny, nz = 200, 201, 202

# 3S+2D goniometer (ID01 goniometer, sample mu, eta, phi detector
# nu, del, mpxy, mpxz
# convention for coordinate system: x downstream; z upwards; y to the
# "outside" (righthanded)
# QConversion will set up the goniometer geometry. So the first argument
# describes the sample rotations, the second the detector rotations and the
# third the primary beam direction.
# For this consider the following right handed coordinate system (feel free to
# use your conventions):
# x: downstream (direction of primary beam)
# y: out of the ring
# z: upwards
# The outer most sample rotation (so the one mounted on the floor) is one
# which turns left-handed (-) around the z-direction -> z- (mu)
# The second sample rotation ('eta') is lefthanded (-) around y -> y-
qconv = xu.experiment.QConversion(['z-', 'y-', 'z-'],
                                  ['z-', 'y-', 'ty', 'tz'],
                                  [1, 0, 0])
hxrd = xu.HXRD([1, 1, 0], [0, 0, 1], qconv=qconv, sampleor='z+')
hxrd._A2QConversion.init_area('z-', 'y+', cch1=333.94, cch2=235.62, Nch1=516,
                              Nch2=516, pwidth1=5.5000e-02, pwidth2=5.5000e-02,
                              distance=0.53588*1000, detrot=-1.495,
                              tiltazimuth=155.0, tilt=0.745, Nav=(2, 2))
# all in mm since mm are used for mpxy,z in the spec-file

qx, qy, qz, gint, gridder = id01.gridmap(s, SCANNR, hxrd, nx, ny, nz)

# ################################################
# for a 3D plot using python function I sugggest
# to use mayavi's mlab package. the basic usage
# is shown below. otherwise have a look at the
# file xrayutilities_export_data2vtk.py in order learn
# how you can get your data to a vtk file for further
# processing.
# #####
# one of the following import statements is needed
# depending on the system/distribution you use
# from mayavi import mlab
# # from enthough.mayavi import mlab
# # plot 3D map using mayavi mlab
# QX,QY,QZ = numpy.mgrid[qx.min():qx.max():1j * nx,
#                        qy.min():qy.max():1j * ny,
#                        qz.min():qz.max():1j * nz]
# INT = xu.maplog(gint,4.5,0)
# mlab.figure()
# mlab.contour3d(QX, QY, QZ, INT, contours=15, opacity=0.5)
# mlab.colorbar(title="log(int)", orientation="vertical")
# mlab.axes(nb_labels=5, xlabel='Qx', ylabel='Qy', zlabel='Qz')
# # mlab.close(all=True)
############################################

# plot 2D sums using matplotlib
plt.figure()
plt.contourf(qx, qy, xu.maplog(gint.sum(axis=2), 2.8, 1.5).T, 50)
plt.xlabel(r"QX ($1/\mathrm{\AA}$)")
plt.ylabel(r"QY ($1/\mathrm{\AA}$)")
plt.colorbar()
# plt.savefig(os.path.join("pics","filename.png"))

# plot 2D slice using matplotlib
plt.figure()
plt.contourf(qx, qy, xu.maplog(gint[:, :, 81:89].sum(axis=2), 3.75, 0).T, 50)
plt.xlabel(r"QX ($1/\mathrm{\AA}$)")
plt.ylabel(r"QY ($1/\mathrm{\AA}$)")
plt.colorbar()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2016 Dominik Kriegner <dominik.kriegner@gmail.com>

import os.path
import unittest

import numpy
import xrayutilities as xu

testfile = '413396'
datadir = os.path.join(os.path.dirname(__file__), 'data')
fullfilename = os.path.join(datadir, testfile)


@unittest.skipIf(not os.path.isfile(fullfilename),
                 "additional test data needed (http://xrayutilities.sf.io)")
class TestIO_numor(unittest.TestCase):
    dshape = (31,)
    dmax = 32823.0
    dmin = 64.0
    motmax = 18.34
    motmin = 16.26
    tpos = 12
    dtpos = 17480.0
    motorname = 'omega'
    countername = 'detector'

    @classmethod
    def setUpClass(cls):
        cls.motor, cls.data = xu.io.numor_scan(testfile, cls.motorname,
                                               path=datadir)
        cls.inte = cls.data[cls.countername]

    def test_datashape(self):
        self.assertEqual(self.dshape, self.data.shape)
        self.assertEqual(self.dshape, self.motor.shape)

    def test_datavalues(self):
        self.assertAlmostEqual(self.motmax, self.motor.max(), places=6)
        self.assertAlmostEqual(self.motmin, self.motor.min(), places=6)
        self.assertAlmostEqual(self.dmax, self.inte.max(), places=6)
        self.assertAlmostEqual(self.dmin, self.inte.min(), places=6)
        self.assertAlmostEqual(self.dtpos, self.inte[self.tpos], places=6)

    def test_equaldata(self):
        self.assertTrue(numpy.all(self.motor == self.data[self.motorname]))


if __name__ == '__main__':
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2016 Dominik Kriegner <dominik.kriegner@gmail.com>

import xrayutilities as xu
from matplotlib.pylab import *

mpl.rcParams['font.size'] = 16.0


en = 8500  # eV
resol = 0.0004  # resolution in q
H, K, L = (1, 1, 1)
qz = linspace(1.8, 2.2, 5000)
Si = xu.materials.Si
hxrd = xu.HXRD(Si.Q(1, 1, -2), Si.Q(1, 1, 1), en=en)

sub = xu.simpack.Layer(Si, inf)
lay = xu.simpack.Layer(xu.materials.SiGe(0.6), 150, relaxation=0.5)
pls = xu.simpack.PseudomorphicStack111('pseudo', sub, lay)

# calculate incidence angle for dynamical diffraction models
qx = hxrd.Transform(Si.Q(H, K, L))[1]
ai = xu.simpack.coplanar_alphai(qx, qz, en)
resolai = abs(xu.simpack.coplanar_alphai(qx, mean(qz) + resol, en) -
              xu.simpack.coplanar_alphai(qx, mean(qz), en))

# comparison of different diffraction models
# simplest kinematical diffraction model
mk = xu.simpack.KinematicalModel(pls, experiment=hxrd, resolution_width=resol)
Ikin = mk.simulate(qz, hkl=(H, K, L), refraction=True)

# simplified dynamical diffraction model
mds = xu.simpack.SimpleDynamicalCoplanarModel(pls, experiment=hxrd,
                                              resolution_width=resolai)
Idynsub = mds.simulate(ai, hkl=(H, K, L), idxref=0)
Idynlay = mds.simulate(ai, hkl=(H, K, L), idxref=1)

# general 2-beam theory based dynamical diffraction model
md = xu.simpack.DynamicalModel(pls, experiment=hxrd, resolution_width=resolai)
Idyn = md.simulate(ai, hkl=(H, K, L))

# plot of calculated intensities
figure('XU-simpack SiGe(111)')
clf()
semilogy(qz, Ikin, label='kinematical')
semilogy(xu.simpack.get_qz(qx, ai, en), Idynsub, label='simpl. dynamical(S)')
semilogy(xu.simpack.get_qz(qx, ai, en), Idynlay, label='simpl. dynamical(L)')
semilogy(xu.simpack.get_qz(qx, ai, en), Idyn, label='full dynamical')
vlines([xu.math.VecNorm(lay.material.Q(H, K, L)) for lay in pls], 1e-9, 1,
       linestyles='dashed')
legend(fontsize='small')
xlim(qz.min(), qz.max())
xlabel(r'Qz ($1/\mathrm{\AA}$)')
ylabel('Intensity (arb. u.)')
tight_layout()
show()
